﻿/**
 * @fileoverview 指定のXML要素の位置に、オブジェクトをコピーペーストし、必要に応じてプレースホルダ文字を置き換える
 * @author Kenshi Muto &lt;kmuto@debian.org&gt;
 * @requires libCommon.jsx
 * @requires glue code.jsx
*/

/*
    Copyright:
    2012-2016 Kenshi Muto <kmuto@debian.org>
----------------------------------------------------------------------
ソフトウェア使用許諾同意書
本ソフトウェアの利用・変更・再配布にあたっては、下記の使用許諾同意書に
同意する必要があります。

1. 本使用許諾同意書における「ソフトウェア」とは、機械可読の資料 (ライブ
   ラリ、スクリプト、ソースファイル、データファイル)、実行形式、および
   文書を意味します。
2. 本ソフトウェアの使用許諾同意書に同意する限りにおいて、使用者は
   本ソフトウェアを自由に利用、変更することができます。
3. 本ソフトウェアに変更を加えない限りにおいて、使用者は本ソフトウェアを
   自由にコピー、再配布することができます。
4. 本ソフトウェアは無保証です。作者およびそれに関連する組織、配布者は、
   本ソフトウェアの使用に起因する一切の直接損害、間接損害、偶発的損害、
   特別損害、懲戒的損害、派生的損害について何らの責任・保証も負いません。
5. 本ソフトウェアを変更した上で再配布するときには、下記の事項すべてに
   従わなければなりません。
   - 使用許諾同意書の内容に変更を加えてはなりません。技術上の理由で
     文字エンコーディングの変換を行うことは許可しますが、その使用者が
     特殊な技術的措置なしに可読な形でなければなりません。
   - 技術上の理由でバイナリ化・難読化を行う場合も、変更箇所を含めた
     ソフトウェアを、その使用者が可読可能な形式の形で同一のメディアで
     提供しなければなりません。本使用許諾同意書の2条および3条により、
     使用者が可読形式の該当ソフトウェアを変更、コピー、再配布することを
     妨げてはなりません。
   - ソフトウェア構成物の所定の作者名の欄に、変更者のクレジット
     (個人名、企業名、所属、連絡先など)を「追加」しなければなりません。
6. 本ソフトウェアを変更した上で再配布するときには、変更理由および
   その内容を明記することが推奨されます。
7. 使用者がソフトウェアに適用可能な特許に対して特許侵害にかかわる何らか
   の行動を開始した時点で、この使用許諾同意書は自動的に終了し、以降
   使用者はこの使用許諾書によって与えられた一切の権利を放棄するものと
   します。

著作権所有者 Copyright (C) 2012-2016 Kenshi Muto. All rights reserved.
使用許諾同意書バージョン1.0
著作権所有者による書面での事前の許可がない限り、この使用許諾同意書
に変更を加えてはなりません。
----------------------------------------------------------------------
*/

#include "../libs/libCommon.jsx"
#include "../libs/glue code.jsx"
#include "../libs/libHandleOverflow2.jsx"

/**
 * XML要素を検索して、指定の要素を発見したらオブジェクトをコピーペーストし、プレースホルダを置き換える
 * @param {Object} obj パラメータオブジェクト
 * @param {String} obj.replaces[[key,val],[key,val]...] プレースホルダの置換設定。keyにプレースホルダキーワード、valに置換する値を指定する。valにnullを指定すると、keyをそのままXML要素の属性名として扱い、その属性値を使用する。このパラメータは破壊されるので、同じ設定を使い回すときでも再設定すること。
 * @param {String} obj.tagname 対象とするXML要素名
 * @param {Object} obj.item コピーペーストするオブジェクト。Group、Rectangle、TextFrameなどのオブジェクトを指定する
 * @param {Document} obj.document ドキュメントオブジェクト (obj.ofobj利用時)
 * @param {CharacterStyle} obj.cstyle 挿入した箇所に適用する文字スタイル(nullの場合は何もしない)
 * @param {Object} obj.ofobj treatOverflow用の設定（nullの場合はページ追加しない）
 * @type Nothing
*/
function processXMLForPlaceGroup(obj) {
  if (obj.replaces == null) obj.replaces = [];
  obj.replacebyattr = [];
  for (var i = 0; i < obj.replaces.length; i++) {
    if (obj.replaces[i][1] == null) {
      obj.replacebyattr.push(obj.replaces[i][0]);
    }
  }
  var myRuleSet = new Array(new processForPlaceGroup(obj));
  __processRuleSet(document.xmlElements.item(0), myRuleSet);
}

/**
 * processXMLForPlaceGroupから呼び出される実体。XML要素を検索して、指定の要素を発見したらオブジェクトをコピーペーストし、プレースホルダを置き換える
 * @param {Object} obj パラメータオブジェクト
 * @param {String} obj.replaces[[key,val],[key,val]...] プレースホルダの置換設定。keyにプレースホルダキーワード、valに置換する値を指定する。valにnullを指定すると、keyをそのままXML要素の属性名として扱い、その属性値を使用する
 * @param {String} obj.replacebyattr[] 属性値置換を使用するプレースホルダkeyの一覧
 * @param {String} obj.tagname 対象とするXML要素名
 * @param {Object} obj.item コピーペーストするオブジェクト。Group、Rectangle、TextFrameなどのオブジェクトを指定する
 * @param {Document} obj.document ドキュメントオブジェクト (obj.ofobj利用時)
 * @param {CharacterStyle} obj.cstyle 挿入した箇所に適用する文字スタイル
 * @param {Object} obj.ofobj treatOverflow用の設定（nullの場合はページ追加しない）
 * @type Nothing
*/
function processForPlaceGroup(obj) {
  this.name = "ProcessSubset";
  if (obj.tagname.match(/^\//) != null) {
    this.xpath = obj.tagname;
  } else {
    this.xpath = "//" + obj.tagname;
  }
  this.apply = function(myElement, myRuleProcessor) {
    obj.point = myElement.storyOffset;
    for (var i = 0; i < obj.replaces.length; i++) {
      for (var i2 = 0; i2 < obj.replacebyattr.length; i2++) {
        if (obj.replacebyattr[i2] == obj.replaces[i][0]) {
          obj.replaces[i][1] = myElement.xmlAttributes.item(obj.replaces[i][0]).value.replace("&lt;", "<").replace("&gt;", ">").replace("&amp;", "&");
          break;
        }
      }
    }
    placeGroup(obj);
    if (obj.ofobj) treatOverflow(obj.document, obj.ofobj);
    return true;
  }
}

/**
 * オブジェクトをコピーペーストし、プレースホルダを置き換える
 * @param {Object} obj パラメータオブジェクト
 * @param {String} obj.replaces[[key,val],[key,val]...] プレースホルダの置換設定。keyにプレースホルダキーワード、valに置換する値を指定する
 * @param {Object} obj.item コピーペーストするオブジェクト。Group、Rectangle、TextFrameなどのオブジェクトを指定する
 * @param {InsertionPoint} obj.point ペーストする位置
 * @param {String} obj.specifiedtfname 明示的なテキストフレーム名
 * @param {CharacterStyle} obj.cstyle 挿入した箇所に適用する文字スタイル
 * @type Nothing
*/
function placeGroup(obj) {
  if (obj.replaces == null) obj.replaces = [];
  var page;
  try {
    page = obj.point.parentTextFrames[0].parentPage;
  } catch (e) {
    page = obj.point.parentTextFrames[0].parent;
  }

  var copieditem = obj.item.duplicate(page);
  for (var i = 0; i < obj.replaces.length; i++) {
    if (obj.specifiedtfname == null) {
      var tf = findStringFromObjectWithGroup(copieditem, obj.replaces[i][0]);
      if (tf[0]) tf[0].contents = obj.replaces[i][1];
    } else {
      for (var j = 0; j < copieditem.allPageItems.length; j++) {
        if (copieditem.allPageItems[j].label == obj.specifiedtfname) {
          copieditem.allPageItems[j].contents = obj.replaces[i][1];
          break;
        }
      }
    }
  }

  copieditem.select();
  app.cut();
  obj.point.select();
  app.paste();
  // 文字スタイル適用
  if (obj.cstyle) obj.point.parent.characters[obj.point.index -1].appliedCharacterStyle = obj.cstyle;
 }
